# -*- coding: utf-8 -*-

import numpy as np

from mirdata import giantsteps_tempo, utils
from tests.test_utils import run_track_tests


def test_track():
    default_trackid = '113'
    data_home = 'tests/resources/mir_datasets/GiantSteps_tempo'
    track = giantsteps_tempo.Track(default_trackid, data_home=data_home)

    expected_attributes = {
        'audio_path': 'tests/resources/mir_datasets/GiantSteps_tempo/audio/28952.LOFI.mp3',
        'annotation_v1_path': 'tests/resources/mir_datasets/GiantSteps_tempo/giantsteps-tempo-dataset'
                              '-0b7d47ba8cae59d3535a02e3db69e2cf6d0af5bb/annotations/jams/28952.LOFI.jams',
        'annotation_v2_path': 'tests/resources/mir_datasets/GiantSteps_tempo/giantsteps-tempo-dataset'
                              '-0b7d47ba8cae59d3535a02e3db69e2cf6d0af5bb/annotations_v2/jams/28952.LOFI.jams',
        'title': '28952',
        'track_id': '113',
    }

    expected_property_types = {
        'tempo': utils.TempoData,
        'tempo_v2': utils.TempoData,
        'genre': str
    }

    run_track_tests(track, expected_attributes, expected_property_types)

    audio, sr = track.audio
    assert sr == 22050, 'sample rate {} is not 22050'.format(sr)
    print(audio.shape)
    assert audio.shape == (2646720,), 'audio shape {} was not (2646720,)'.format(
        audio.shape
    )


def test_load_genre():
    genre_path = 'tests/resources/mir_datasets/GiantSteps_tempo/giantsteps-tempo-dataset' \
                 '-0b7d47ba8cae59d3535a02e3db69e2cf6d0af5bb/annotations/jams/28952.LOFI.jams'

    genre_data = giantsteps_tempo.load_genre(genre_path)

    assert type(genre_data) == str

    assert genre_data == "trance"

    assert giantsteps_tempo.load_genre(None) is None


def test_load_tempo():
    tempo_path = (
        'tests/resources/mir_datasets/GiantSteps_tempo/giantsteps-tempo-dataset'
        '-0b7d47ba8cae59d3535a02e3db69e2cf6d0af5bb/annotations/jams/28952.LOFI.jams'
    )
    tempo_data = giantsteps_tempo.load_tempo(tempo_path)

    assert type(tempo_data) == utils.TempoData

    assert tempo_data == utils.TempoData(time=np.array([0.0]), duration=np.array([120.0]),
                                         value=np.array([137.6]), confidence=np.array([1.0]))

    tempo_path = (
        'tests/resources/mir_datasets/GiantSteps_tempo/giantsteps-tempo-dataset'
        '-0b7d47ba8cae59d3535a02e3db69e2cf6d0af5bb/annotations_v2/jams/28952.LOFI.jams'
    )
    tempo_data = giantsteps_tempo.load_tempo(tempo_path)

    assert type(tempo_data) == utils.TempoData

    assert np.array_equal(tempo_data.time, np.array([0., 0.]))
    assert np.array_equal(tempo_data.duration, np.array([120., 120.]))
    assert np.array_equal(tempo_data.value, np.array([77., 139.]))
    assert giantsteps_tempo.load_tempo(None) is None



# -*- coding: utf-8 -*-

import os
import shutil
import sys

from mirdata import download_utils

import pytest

if sys.version_info.major == 3:
    builtin_module_name = 'builtins'
    from pathlib import Path
else:
    builtin_module_name = '__builtin__'
    from pathlib2 import Path


@pytest.fixture
def mock_file(mocker):
    return mocker.patch.object(download_utils, 'download_from_remote')


@pytest.fixture
def mock_downloader(mocker):
    return mocker.patch.object(download_utils, 'downloader')


@pytest.fixture
def mock_untar(mocker):
    return mocker.patch.object(download_utils, 'untar')


@pytest.fixture
def mock_unzip(mocker):
    return mocker.patch.object(download_utils, 'unzip')


@pytest.fixture
def mock_path(mocker, mock_file):
    return mocker.patch.object(Path, 'mkdir')


def test_downloader(mocker, mock_path, capsys):
    mock_zip = mocker.patch.object(download_utils, 'download_zip_file')
    mock_tar = mocker.patch.object(download_utils, 'download_tar_file')
    mock_file = mocker.patch.object(download_utils, 'download_from_remote')

    zip_remote = download_utils.RemoteFileMetadata(
        filename='remote.zip', url='a', checksum=('1234'), destination_dir=None
    )
    tar_remote = download_utils.RemoteFileMetadata(
        filename='remote.tar.gz', url='a', checksum=('1234'), destination_dir=None
    )

    file_remote = download_utils.RemoteFileMetadata(
        filename='remote.txt', url='a', checksum=('1234'), destination_dir=None
    )

    # Zip only
    download_utils.downloader('a', remotes={'b': zip_remote})
    mock_zip.assert_called_once_with(zip_remote, 'a', False, True)
    mocker.resetall()

    # tar only
    download_utils.downloader('a', remotes={'b': tar_remote})
    mock_tar.assert_called_once_with(tar_remote, 'a', False, True)
    mocker.resetall()

    # file only
    download_utils.downloader('a', remotes={'b': file_remote})
    mock_file.assert_called_once_with(file_remote, 'a', False)
    mocker.resetall()

    # zip and tar
    download_utils.downloader('a', remotes={'b': zip_remote, 'c': tar_remote})
    mock_zip.assert_called_once_with(zip_remote, 'a', False, True)
    mock_tar.assert_called_once_with(tar_remote, 'a', False, True)
    mocker.resetall()

    # zip and file
    download_utils.downloader('a', remotes={'b': zip_remote, 'c': file_remote})
    mock_zip.assert_called_once_with(zip_remote, 'a', False, True)
    mock_file.assert_called_once_with(file_remote, 'a', False)
    mocker.resetall()

    # tar and file
    download_utils.downloader('a', remotes={'b': tar_remote, 'c': file_remote})
    mock_tar.assert_called_once_with(tar_remote, 'a', False, True)
    mock_file.assert_called_once_with(file_remote, 'a', False)
    mocker.resetall()

    # zip and tar and file
    download_utils.downloader(
        'a', remotes={'b': zip_remote, 'c': tar_remote, 'd': file_remote}
    )
    mock_zip.assert_called_once_with(zip_remote, 'a', False, True)
    mock_file.assert_called_once_with(file_remote, 'a', False)
    mock_tar.assert_called_once_with(tar_remote, 'a', False, True)
    mocker.resetall()

    # test partial download
    download_utils.downloader(
        'a',
        remotes={'b': zip_remote, 'c': tar_remote, 'd': file_remote},
        partial_download=['b', 'd'],
    )
    mock_zip.assert_called_once_with(zip_remote, 'a', False, True)
    mock_file.assert_called_once_with(file_remote, 'a', False)
    mocker.resetall()

    # test bad type partial download
    with pytest.raises(ValueError):
        download_utils.downloader(
            'a',
            remotes={'b': zip_remote, 'c': tar_remote, 'd': file_remote},
            partial_download='b',
        )

    with pytest.raises(ValueError):
        download_utils.downloader(
            'a',
            remotes={'b': zip_remote, 'c': tar_remote, 'd': file_remote},
            partial_download=['d', 'e'],
        )

    # test info message
    captured = capsys.readouterr()  # skip everything printed before this
    download_utils.downloader('a', info_message='I am a message!')
    captured = capsys.readouterr()
    assert captured.out == "I am a message!\n"


def test_download_from_remote(httpserver, tmpdir):
    httpserver.serve_content(open('tests/resources/remote.wav').read())

    TEST_REMOTE = download_utils.RemoteFileMetadata(
        filename='remote.wav',
        url=httpserver.url,
        checksum=('3f77d0d69dc41b3696f074ad6bf2852f'),
        destination_dir=None,
    )

    download_path = download_utils.download_from_remote(TEST_REMOTE, str(tmpdir))
    expected_download_path = os.path.join(str(tmpdir), 'remote.wav')
    assert expected_download_path == download_path


def test_download_from_remote_destdir(httpserver, tmpdir):
    httpserver.serve_content(open('tests/resources/remote.wav').read())

    TEST_REMOTE = download_utils.RemoteFileMetadata(
        filename='remote.wav',
        url=httpserver.url,
        checksum=('3f77d0d69dc41b3696f074ad6bf2852f'),
        destination_dir='subfolder',
    )

    download_path = download_utils.download_from_remote(TEST_REMOTE, str(tmpdir))
    expected_download_path = os.path.join(str(tmpdir), 'subfolder', 'remote.wav')
    assert expected_download_path == download_path


def test_download_from_remote_raises_IOError(httpserver, tmpdir):
    httpserver.serve_content('File not found!', 404)

    TEST_REMOTE = download_utils.RemoteFileMetadata(
        filename='remote.wav',
        url=httpserver.url,
        checksum=('1234'),
        destination_dir=None,
    )

    with pytest.raises(IOError):
        download_utils.download_from_remote(TEST_REMOTE, str(tmpdir))


def test_unzip():
    download_utils.unzip('tests/resources/file.zip', cleanup=False)
    expected_file_location = os.path.join('tests', 'resources', 'file.txt')
    assert os.path.exists(expected_file_location)
    os.remove(expected_file_location)


def test_untar():
    download_utils.untar('tests/resources/file.tar.gz', cleanup=False)
    expected_file_location = os.path.join('tests', 'resources', 'file', 'file.txt')
    assert os.path.exists(expected_file_location)
    os.remove(expected_file_location)


def test_download_zip_file(mocker, mock_file, mock_unzip):
    mock_file.return_value = "foo"
    download_utils.download_zip_file("a", "b", True)

    mock_file.assert_called_once_with("a", "b", True)
    mock_unzip.assert_called_once_with("foo", cleanup=True)
    if os.path.exists('a'):
        shutil.rmtree('a')


def test_download_tar_file(mocker, mock_file, mock_untar):
    mock_file.return_value = "foo"
    download_utils.download_tar_file("a", "b", True)

    mock_file.assert_called_once_with("a", "b", True)
    mock_untar.assert_called_once_with("foo", cleanup=True)
    if os.path.exists('a'):
        shutil.rmtree('a')



# -*- coding: utf-8 -*-

import os

from tests.test_utils import run_track_tests

from mirdata import gtzan_genre
from tests.test_utils import DEFAULT_DATA_HOME

TEST_DATA_HOME = "tests/resources/mir_datasets/GTZAN-Genre"


def test_track_default_data_home():
    # test data home None
    track_default = gtzan_genre.Track("country.00000")
    assert track_default._data_home == os.path.join(DEFAULT_DATA_HOME, "GTZAN-Genre")


def test_track():
    default_trackid = "country.00000"
    track = gtzan_genre.Track(default_trackid, data_home=TEST_DATA_HOME)
    expected_attributes = {
        'genre': "country",
        'audio_path': "tests/resources/mir_datasets/GTZAN-Genre/"
        + "gtzan_genre/genres/country/country.00000.wav",
        'track_id': "country.00000",
    }
    run_track_tests(track, expected_attributes, {})

    audio, sr = track.audio
    assert sr == 22050
    assert audio.shape == (663300,)


def test_hiphop():
    track = gtzan_genre.Track('hiphop.00000', data_home=TEST_DATA_HOME)
    assert track.genre == 'hip-hop'


def test_to_jams():
    default_trackid = "country.00000"
    track = gtzan_genre.Track(default_trackid, data_home=TEST_DATA_HOME)
    jam = track.to_jams()

    # Validate GTZAN schema
    assert jam.validate()

    # Test the that the genre parser of mirdata is correct
    assert jam.annotations["tag_gtzan"][0].data[0].value == "country"



# -*- coding: utf-8 -*-

import numpy as np

from mirdata import medleydb_pitch, utils
from tests.test_utils import run_track_tests


def test_track():
    default_trackid = 'AClassicEducation_NightOwl_STEM_08'
    data_home = 'tests/resources/mir_datasets/MedleyDB-Pitch'
    track = medleydb_pitch.Track(default_trackid, data_home=data_home)

    expected_attributes = {
        'track_id': 'AClassicEducation_NightOwl_STEM_08',
        'audio_path': 'tests/resources/mir_datasets/'
        + 'MedleyDB-Pitch/audio/AClassicEducation_NightOwl_STEM_08.wav',
        'pitch_path': 'tests/resources/mir_datasets/'
        + 'MedleyDB-Pitch/pitch/AClassicEducation_NightOwl_STEM_08.csv',
        'instrument': 'male singer',
        'artist': 'AClassicEducation',
        'title': 'NightOwl',
        'genre': 'Singer/Songwriter',
    }

    expected_property_types = {'pitch': utils.F0Data}

    run_track_tests(track, expected_attributes, expected_property_types)

    y, sr = track.audio
    assert sr == 44100
    assert y.shape == (44100 * 2,)


def test_to_jams():

    data_home = 'tests/resources/mir_datasets/MedleyDB-Pitch'
    track = medleydb_pitch.Track(
        'AClassicEducation_NightOwl_STEM_08', data_home=data_home
    )
    jam = track.to_jams()

    f0s = jam.search(namespace='pitch_contour')[0]['data']
    assert [f0.time for f0 in f0s] == [0.06965986394557823, 0.07546485260770976]
    assert [f0.duration for f0 in f0s] == [0.0, 0.0]
    assert [f0.value for f0 in f0s] == [
        {'frequency': 0.0, 'index': 0, 'voiced': False},
        {'frequency': 191.877, 'index': 0, 'voiced': True},
    ]
    assert [f0.confidence for f0 in f0s] == [0.0, 1.0]

    assert jam['file_metadata']['title'] == 'NightOwl'
    assert jam['file_metadata']['artist'] == 'AClassicEducation'


def test_load_pitch():
    # load a file which exists
    pitch_path = (
        'tests/resources/mir_datasets/MedleyDB-Pitch/'
        + 'pitch/AClassicEducation_NightOwl_STEM_08.csv'
    )
    pitch_data = medleydb_pitch.load_pitch(pitch_path)

    # check types
    assert type(pitch_data) == utils.F0Data
    assert type(pitch_data.times) is np.ndarray
    assert type(pitch_data.frequencies) is np.ndarray
    assert type(pitch_data.confidence) is np.ndarray

    # check values
    assert np.array_equal(
        pitch_data.times, np.array([0.06965986394557823, 0.07546485260770976])
    )
    assert np.array_equal(pitch_data.frequencies, np.array([0.0, 191.877]))
    assert np.array_equal(pitch_data.confidence, np.array([0.0, 1.0]))


def test_load_metadata():
    data_home = 'tests/resources/mir_datasets/MedleyDB-Pitch'
    metadata = medleydb_pitch._load_metadata(data_home)
    assert metadata['data_home'] == data_home
    assert metadata['AClassicEducation_NightOwl_STEM_08'] == {
        'audio_path': 'MedleyDB-Pitch/audio/AClassicEducation_NightOwl_STEM_08.wav',
        'pitch_path': 'MedleyDB-Pitch/pitch/AClassicEducation_NightOwl_STEM_08.csv',
        'instrument': 'male singer',
        'artist': 'AClassicEducation',
        'title': 'NightOwl',
        'genre': 'Singer/Songwriter',
    }



# -*- coding: utf-8 -*-

import numpy as np

from mirdata import rwc_classical, utils
from tests.test_utils import run_track_tests


def test_track():
    default_trackid = 'RM-C003'
    data_home = 'tests/resources/mir_datasets/RWC-Classical'
    track = rwc_classical.Track(default_trackid, data_home=data_home)

    expected_attributes = {
        'track_id': 'RM-C003',
        'audio_path': 'tests/resources/mir_datasets/RWC-Classical/'
        + 'audio/rwc-c-m01/3.wav',
        'sections_path': 'tests/resources/mir_datasets/RWC-Classical/'
        + 'annotations/AIST.RWC-MDB-C-2001.CHORUS/RM-C003.CHORUS.TXT',
        'beats_path': 'tests/resources/mir_datasets/RWC-Classical/'
        + 'annotations/AIST.RWC-MDB-C-2001.BEAT/RM-C003.BEAT.TXT',
        'piece_number': 'No. 3',
        'suffix': 'M01',
        'track_number': 'Tr. 03',
        'title': 'Symphony no.5 in C minor, op.67. 1st mvmt.',
        'composer': 'Beethoven, Ludwig van',
        'artist': 'Tokyo City Philharmonic Orchestra',
        'duration': 435,
        'category': 'Symphony',
    }

    expected_property_types = {'beats': utils.BeatData, 'sections': utils.SectionData}

    run_track_tests(track, expected_attributes, expected_property_types)

    # test audio loading functions
    y, sr = track.audio
    assert sr == 44100
    assert y.shape == (44100 * 2,)


def test_to_jams():

    data_home = 'tests/resources/mir_datasets/RWC-Classical'
    track = rwc_classical.Track('RM-C003', data_home=data_home)
    jam = track.to_jams()

    beats = jam.search(namespace='beat')[0]['data']
    assert [beat.time for beat in beats] == [
        1.65,
        2.58,
        2.95,
        3.33,
        3.71,
        4.09,
        5.18,
        6.28,
    ]
    assert [beat.duration for beat in beats] == [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
    assert [beat.value for beat in beats] == [2, 1, 2, 1, 2, 1, 2, 1]
    assert [beat.confidence for beat in beats] == [
        None,
        None,
        None,
        None,
        None,
        None,
        None,
        None,
    ]

    segments = jam.search(namespace='segment')[0]['data']
    assert [segment.time for segment in segments] == [0.29, 419.96]
    assert [segment.duration for segment in segments] == [45.85, 13.75]
    assert [segment.value for segment in segments] == ['chorus A', 'ending']
    assert [segment.confidence for segment in segments] == [None, None]

    assert jam['file_metadata']['title'] == 'Symphony no.5 in C minor, op.67. 1st mvmt.'
    assert jam['file_metadata']['artist'] == 'Tokyo City Philharmonic Orchestra'


def test_load_sections():
    # load a file which exists
    section_path = (
        'tests/resources/mir_datasets/RWC-Classical/'
        + 'annotations/AIST.RWC-MDB-C-2001.CHORUS/RM-C003.CHORUS.TXT'
    )
    section_data = rwc_classical.load_sections(section_path)

    # check types
    assert type(section_data) == utils.SectionData
    assert type(section_data.intervals) is np.ndarray
    assert type(section_data.labels) is list

    # check values
    assert np.array_equal(section_data.intervals[:, 0], np.array([0.29, 419.96]))
    assert np.array_equal(section_data.intervals[:, 1], np.array([46.14, 433.71]))
    assert np.array_equal(section_data.labels, np.array(['chorus A', 'ending']))


def test_position_in_bar():
    beat_positions1 = np.array([48, 384, 48, 384, 48, 384, 48, 384])
    times1 = np.array([1, 2, 3, 4, 5, 6, 7, 8])
    fixed_positions1 = np.array([2, 1, 2, 1, 2, 1, 2, 1])
    fixed_times1 = np.array([1, 2, 3, 4, 5, 6, 7, 8])
    actual_positions1, actual_times1 = rwc_classical._position_in_bar(
        beat_positions1, times1
    )
    assert np.array_equal(actual_positions1, fixed_positions1)
    assert np.array_equal(actual_times1, fixed_times1)

    beat_positions2 = np.array([-1, 48, 384, 48, 384, 48, 384, 48])
    times2 = np.array([1, 2, 3, 4, 5, 6, 7, 8])
    fixed_positions2 = np.array([2, 1, 2, 1, 2, 1, 2])
    fixed_times2 = np.array([2, 3, 4, 5, 6, 7, 8])
    actual_positions2, actual_times2 = rwc_classical._position_in_bar(
        beat_positions2, times2
    )
    assert np.array_equal(actual_positions2, fixed_positions2)
    assert np.array_equal(actual_times2, fixed_times2)

    beat_positions3 = np.array([384, 48, 384, 48, 384, 48, 384, 48, 384])
    times3 = np.array([1, 2, 3, 4, 5, 6, 7, 8, 9])
    fixed_positions3 = np.array([1, 2, 1, 2, 1, 2, 1, 2, 1])
    fixed_times3 = np.array([1, 2, 3, 4, 5, 6, 7, 8, 9])
    actual_positions3, actual_times3 = rwc_classical._position_in_bar(
        beat_positions3, times3
    )
    assert np.array_equal(actual_positions3, fixed_positions3)
    assert np.array_equal(actual_times3, fixed_times3)

    beat_positions4 = np.array([48, 12, 24, 36, 48, 12, 24, 36])
    times4 = np.array([1, 2, 3, 4, 5, 6, 7, 8])
    fixed_positions4 = np.array([1, 2, 3, 4, 1, 2, 3, 4])
    fixed_times4 = np.array([1, 2, 3, 4, 5, 6, 7, 8])
    actual_positions4, actual_times4 = rwc_classical._position_in_bar(
        beat_positions4, times4
    )
    assert np.array_equal(actual_positions4, fixed_positions4)
    assert np.array_equal(actual_times4, fixed_times4)

    beat_positions5 = np.array([24, 36, 48, 12, 24, 36, 48])
    times5 = np.array([1, 2, 3, 4, 5, 6, 7])
    fixed_positions5 = np.array([3, 4, 1, 2, 3, 4, 1])
    fixed_times5 = np.array([1, 2, 3, 4, 5, 6, 7])
    actual_positions5, actual_times5 = rwc_classical._position_in_bar(
        beat_positions5, times5
    )
    assert np.array_equal(actual_positions5, fixed_positions5)
    assert np.array_equal(actual_times5, fixed_times5)


def test_load_beats():
    beats_path = (
        'tests/resources/mir_datasets/RWC-Classical/'
        + 'annotations/AIST.RWC-MDB-C-2001.BEAT/RM-C003.BEAT.TXT'
    )
    beat_data = rwc_classical.load_beats(beats_path)

    # check types
    assert type(beat_data) is utils.BeatData
    assert type(beat_data.beat_times) is np.ndarray
    assert type(beat_data.beat_positions) is np.ndarray

    # check values
    assert np.array_equal(
        beat_data.beat_times, np.array([1.65, 2.58, 2.95, 3.33, 3.71, 4.09, 5.18, 6.28])
    )
    assert np.array_equal(beat_data.beat_positions, np.array([2, 1, 2, 1, 2, 1, 2, 1]))


def test_load_metadata():
    data_home = 'tests/resources/mir_datasets/RWC-Classical'
    metadata = rwc_classical._load_metadata(data_home)
    assert metadata['data_home'] == data_home
    assert metadata['RM-C003'] == {
        'piece_number': 'No. 3',
        'suffix': 'M01',
        'track_number': 'Tr. 03',
        'title': 'Symphony no.5 in C minor, op.67. 1st mvmt.',
        'composer': 'Beethoven, Ludwig van',
        'artist': 'Tokyo City Philharmonic Orchestra',
        'duration': 435,
        'category': 'Symphony',
    }



# -*- coding: utf-8 -*-
"""track object utility functions
"""
import types

import numpy as np

MAX_STR_LEN = 100


class Track(object):
    def __repr__(self):
        properties = [v for v in dir(self.__class__) if not v.startswith("_")]
        attributes = [
            v for v in dir(self) if not v.startswith("_") and v not in properties
        ]

        repr_str = "Track(\n"

        for attr in attributes:
            val = getattr(self, attr)
            if isinstance(val, str):
                if len(val) > MAX_STR_LEN:
                    val = "...{}".format(val[-MAX_STR_LEN:])
                val = '"{}"'.format(val)
            repr_str += "  {}={},\n".format(attr, val)

        for prop in properties:
            val = getattr(self.__class__, prop)
            if isinstance(val, types.FunctionType):
                continue

            if val.__doc__ is None:
                raise ValueError("{} has no documentation".format(prop))

            val_type_str = val.__doc__.split(":")[0]
            repr_str += "  {}: {},\n".format(prop, val_type_str)

        repr_str += ")"
        return repr_str

    def to_jams(self):
        raise NotImplementedError


class MultiTrack(Track):
    """MultiTrack class.

    A multitrack class is a collection of track objects and their associated audio
    that can be mixed together.
    A multitrack is iteslf a Track, and can have its own associated audio (such as
    a mastered mix), its own metadata and its own annotations.

    """

    def _check_mixable(self):
        if not hasattr(self, "tracks") or not hasattr(self, "track_audio_property"):
            raise NotImplementedError(
                "This MultiTrack has no tracks/track_audio_property. Cannot perform mixing"
            )

    def get_target(self, track_keys, weights=None, average=True, enforce_length=True):
        """Get target which is a linear mixture of tracks

        Args:
            track_keys (list): list of track keys to mix together
            weights (list or None): list of positive scalars to be used in the average
            average (bool): if True, computes a weighted average of the tracks
                if False, computes a weighted sum of the tracks
            enforce_length (bool): If True, raises ValueError if the tracks are 
                not the same length. If False, pads audio with zeros to match the length
                of the longest track
        
        Returns:
            target (np.ndarray): target audio with shape (n_channels, n_samples)

        Raises:
            ValueError: 
                if sample rates of the tracks are not equal
                if enforce_length=True and lengths are not equal

        """
        self._check_mixable()
        signals = []
        lengths = []
        sample_rates = []
        for k in track_keys:
            audio, sample_rate = getattr(self.tracks[k], self.track_audio_property)
            # ensure all signals are shape (n_channels, n_samples)
            if len(audio.shape) == 1:
                audio = audio[np.newaxis, :]
            signals.append(audio)
            lengths.append(audio.shape[1])
            sample_rates.append(sample_rate)

        if len(set(sample_rates)) > 1:
            raise ValueError(
                "Sample rates for tracks {} are not equal: {}".format(
                    track_keys, sample_rates
                )
            )

        max_length = np.max(lengths)
        if any([l != max_length for l in lengths]):
            if enforce_length:
                raise ValueError(
                    "Track's {} audio are not the same length {}. Use enforce_length=False to pad with zeros.".format(
                        track_keys, lengths
                    )
                )
            else:
                # pad signals to the max length
                signals = [
                    np.pad(signal, ((0, 0), (0, max_length - signal.shape[1])))
                    for signal in signals
                ]

        if weights is None:
            weights = np.ones((len(track_keys),))

        target = np.average(signals, axis=0, weights=weights)
        if not average:
            target *= np.sum(weights)

        return target

    def get_random_target(self, n_tracks=None, min_weight=0.3, max_weight=1.0):
        """Get a random target by combining a random selection of tracks with random weights

        Args:
            n_tracks (int or None): number of tracks to randomly mix. If None, uses all tracks
            min_weight (float): minimum possible weight when mixing
            max_weight (float): maximum possible weight when mixing
        
        Returns:
            target (np.ndarray): mixture audio with shape (n_samples, n_channels)
            tracks (list): list of keys of included tracks
            weights (list): list of weights used to mix tracks
        """
        self._check_mixable()
        tracks = list(self.tracks.keys())
        if n_tracks is not None and n_tracks < len(tracks):
            tracks = np.random.choice(tracks, n_tracks, replace=False)

        weights = np.random.uniform(low=min_weight, high=max_weight, size=len(tracks))
        target = self.get_target(tracks, weights=weights)
        return target, tracks, weights

    def get_mix(self):
        """Create a linear mixture given a subset of tracks.

        Args:
            track_keys (list): list of track keys to mix together
        
        Returns:
            target (np.ndarray): mixture audio with shape (n_samples, n_channels)
        """
        self._check_mixable()
        return self.get_target(list(self.tracks.keys()))



""" Setup script for mir_datasets. """
from importlib.machinery import SourceFileLoader
from setuptools import setup, find_packages

version_sfl = SourceFileLoader("mirdata.version", "mirdata/version.py")
version_module = version_sfl.load_module()

with open("README.md", "r") as fdesc:
    long_description = fdesc.read()

if __name__ == "__main__":
    setup(
        name="mirdata",
        version=version_module.version,
        description="Common loaders for MIR datasets.",
        url="https://github.com/mir-dataset-loaders/mirdata",
        packages=find_packages(exclude=["test", "*.test", "*.test.*"]),
        download_url="http://github.com/mir-dataset-loaders/mirdata/releases",
        package_data={"mirdata": ["indexes/*"]},
        long_description=long_description,
        long_description_content_type="text/markdown",
        classifiers=[
            "License :: OSI Approved :: BSD License",
            "Programming Language :: Python",
            "Development Status :: 3 - Alpha",
            "Intended Audience :: Science/Research",
            "Topic :: Multimedia :: Sound/Audio :: Analysis",
            "Programming Language :: Python :: 3.6",
            "Programming Language :: Python :: 3.7",
        ],
        keywords="mir dataset loader audio",
        license="BSD-3-Clause",
        install_requires=[
            "tqdm",
            "librosa >= 0.8.0",
            "numpy>=1.16",
            "jams",
            "requests",
            "pretty_midi >= 0.2.8",
        ],
        extras_require={
            "tests": [
                "pytest>=4.4.0",
                "pytest-cov>=2.6.1",
                "pytest-pep8>=1.0.0",
                "pytest-mock>=1.10.1",
                "pytest-localserver>=0.5.0",
                "testcontainers>=2.3",
                "future==0.17.1",
                "coveralls>=1.7.0",
            ],
            "docs": [
                "dali-dataset==1.1",
                "numpydoc",
                "recommonmark",
                "sphinx",
                "sphinxcontrib-napoleon",
                "sphinx_rtd_theme",
            ],
            "dali": ["dali-dataset==1.1"],
        },
    )



# -*- coding: utf-8 -*-

import numpy as np

from mirdata import beatport_key, utils
from tests.test_utils import run_track_tests


def test_track():
    default_trackid = '0'
    data_home = 'tests/resources/mir_datasets/beatport_key'
    track = beatport_key.Track(default_trackid, data_home=data_home)

    expected_attributes = {
        'audio_path': 'tests/resources/mir_datasets/beatport_key/audio/100066 Lindstrom - Monsteer (Original Mix).mp3',
        'keys_path': 'tests/resources/mir_datasets/beatport_key/keys/100066 Lindstrom - Monsteer (Original Mix).txt',
        'metadata_path': 'tests/resources/mir_datasets/beatport_key/meta/100066 Lindstrom - Monsteer (Original Mix).json',
        'title': '100066 Lindstrom - Monsteer (Original Mix)',
        'track_id': '0',
    }

    expected_property_types = {
        'key': str,
        'genres': dict,
        'artists': list,
        'tempo': int
    }

    run_track_tests(track, expected_attributes, expected_property_types)

    audio, sr = track.audio
    assert sr == 44100, 'sample rate {} is not 44100'.format(sr)
    assert audio.shape == (5292000,), 'audio shape {} was not (5292000,)'.format(
        audio.shape
    )


def test_to_jams():
    data_home = 'tests/resources/mir_datasets/beatport_key'
    track = beatport_key.Track('0', data_home=data_home)
    jam = track.to_jams()
    assert (
            jam['sandbox']['key'] == 'D minor'
    ), 'key does not match expected'

    assert (
            jam['file_metadata']['title'] == '100066 Lindstrom - Monsteer (Original Mix)'
    ), 'title does not match expected'
    sand_box = {
        "artists": [
            "Lindstrom"
        ],
        "genres": {
            "genres": [
                "Electronica / Downtempo"
            ],
            "sub_genres": []
        },
        "tempo": 115,
        "key": "D minor"
    }
    assert (
            dict(jam['sandbox']) == sand_box
    ), 'sandbox does not match expected'


def test_load_key():
    key_path = (
        'tests/resources/mir_datasets/beatport_key/keys/100066 Lindstrom - Monsteer (Original Mix).txt'
    )
    key_data = beatport_key.load_key(key_path)

    assert type(key_data) == str

    assert key_data == "D minor"

    assert beatport_key.load_key(None) is None


def test_load_meta():
    meta_path = (
        'tests/resources/mir_datasets/beatport_key/meta/100066 Lindstrom - Monsteer (Original Mix).json'
    )
    genres = {'genres': ['Electronica / Downtempo'], 'sub_genres': []}
    artists = ['Lindstrom']
    tempo = 115

    assert type(beatport_key.load_genre(meta_path)) == dict
    assert type(beatport_key.load_artist(meta_path)) == list
    assert type(beatport_key.load_tempo(meta_path)) == int

    assert beatport_key.load_genre(meta_path) == genres
    assert beatport_key.load_artist(meta_path) == artists
    assert beatport_key.load_tempo(meta_path) == tempo

    assert beatport_key.load_genre(None) is None
    assert beatport_key.load_artist(None) is None
    assert beatport_key.load_tempo(None) is None


def test_find_replace():
    with open('tests/resources/mir_datasets/beatport_key/find_replace.json', 'w') as the_file:
        the_file.write('{"probando": nan}')
    beatport_key.find_replace('tests/resources/mir_datasets/beatport_key', ": nan", ": null", "*.json")
    f = open('tests/resources/mir_datasets/beatport_key/find_replace.json', "r")
    content = f.read()
    assert content == '{"probando": null}'



# -*- coding: utf-8 -*-
import os
import shutil
import pretty_midi
import numpy as np

from mirdata import maestro, utils, download_utils
from tests.test_utils import run_track_tests


def test_track():
    default_trackid = '2018/MIDI-Unprocessed_Chamber3_MID--AUDIO_10_R3_2018_wav--1'
    data_home = 'tests/resources/mir_datasets/MAESTRO'
    track = maestro.Track(default_trackid, data_home=data_home)

    expected_attributes = {
        'track_id': '2018/MIDI-Unprocessed_Chamber3_MID--AUDIO_10_R3_2018_wav--1',
        'midi_path': os.path.join(
            data_home,
            '2018/MIDI-Unprocessed_Chamber3_MID--AUDIO_10_R3_2018_wav--1.midi',
        ),
        'audio_path': os.path.join(
            data_home, '2018/MIDI-Unprocessed_Chamber3_MID--AUDIO_10_R3_2018_wav--1.wav'
        ),
        'canonical_composer': 'Alban Berg',
        'canonical_title': 'Sonata Op. 1',
        'year': 2018,
        'duration': 698.661160312,
        'split': 'train',
    }

    expected_property_types = {'notes': utils.NoteData, 'midi': pretty_midi.PrettyMIDI}

    assert track._track_paths == {
        'audio': [
            '2018/MIDI-Unprocessed_Chamber3_MID--AUDIO_10_R3_2018_wav--1.wav',
            '1694d8431f01eeb2a18444196550b99d',
        ],
        'midi': [
            '2018/MIDI-Unprocessed_Chamber3_MID--AUDIO_10_R3_2018_wav--1.midi',
            '4901b1578ee4fe8c1696e02f60924949',
        ],
    }

    run_track_tests(track, expected_attributes, expected_property_types)

    # test audio loading functions
    audio, sr = track.audio
    assert sr == 48000
    assert audio.shape == (48000 * 2,)


def test_load_midi():
    midi_file = (
        'tests/resources/mir_datasets/MAESTRO/2018/'
        + 'MIDI-Unprocessed_Chamber3_MID--AUDIO_10_R3_2018_wav--1.midi'
    )
    midi = maestro.load_midi(midi_file)
    assert len(midi.instruments) == 1
    assert len(midi.instruments[0].notes) == 4197


def test_load_notes():
    midi_file = (
        'tests/resources/mir_datasets/MAESTRO/2018/'
        + 'MIDI-Unprocessed_Chamber3_MID--AUDIO_10_R3_2018_wav--1.midi'
    )
    notes = maestro.load_notes(midi_file)
    expected_intervals = np.array([[0.98307292, 1.80989583], [1.78385417, 1.90625]])
    assert np.allclose(notes.intervals[0:2], expected_intervals)
    assert np.allclose(notes.notes[0:2], np.array([391.99543598, 523.2511306]))
    assert np.array_equal(notes.confidence[0:2], np.array([52, 67]))


def test_load_metadata():
    data_home = 'tests/resources/mir_datasets/MAESTRO'
    metadata = maestro._load_metadata(data_home)
    default_trackid = '2018/MIDI-Unprocessed_Chamber3_MID--AUDIO_10_R3_2018_wav--1'

    assert metadata['data_home'] == data_home
    assert metadata[default_trackid] == {
        "canonical_composer": "Alban Berg",
        "canonical_title": "Sonata Op. 1",
        "split": "train",
        "year": 2018,
        "midi_filename": "2018/MIDI-Unprocessed_Chamber3_MID--AUDIO_10_R3_2018_wav--1.midi",
        "audio_filename": "2018/MIDI-Unprocessed_Chamber3_MID--AUDIO_10_R3_2018_wav--1.wav",
        "duration": 698.661160312,
    }
    metadata_none = maestro._load_metadata('asdf/asdf')
    assert metadata_none is None


def test_download_partial(httpserver):
    data_home = 'tests/resources/mir_datasets/MAESTRO_download'
    if os.path.exists(data_home):
        shutil.rmtree(data_home)

    httpserver.serve_content(
        open('tests/resources/download/maestro-v2.0.0.json', 'r').read()
    )
    maestro.REMOTES = {
        'all': download_utils.RemoteFileMetadata(
            filename='1-maestro-v2.0.0.json',
            url=httpserver.url,
            checksum=('d41d8cd98f00b204e9800998ecf8427e'),
            destination_dir=None,
        ),
        'midi': download_utils.RemoteFileMetadata(
            filename='2-maestro-v2.0.0.json',
            url=httpserver.url,
            checksum=('d41d8cd98f00b204e9800998ecf8427e'),
            destination_dir=None,
        ),
        'metadata': download_utils.RemoteFileMetadata(
            filename='3-maestro-v2.0.0.json',
            url=httpserver.url,
            checksum=('d41d8cd98f00b204e9800998ecf8427e'),
            destination_dir='maestro-v2.0.0',
        ),
    }
    maestro.download(data_home=data_home, partial_download=None)
    assert os.path.exists(os.path.join(data_home, '1-maestro-v2.0.0.json'))
    assert not os.path.exists(os.path.join(data_home, '2-maestro-v2.0.0.json'))
    assert not os.path.exists(os.path.join(data_home, '3-maestro-v2.0.0.json'))

    if os.path.exists(data_home):
        shutil.rmtree(data_home)
    maestro.download(data_home=data_home, partial_download=['all', 'midi'])
    assert os.path.exists(os.path.join(data_home, '1-maestro-v2.0.0.json'))
    assert not os.path.exists(os.path.join(data_home, '2-maestro-v2.0.0.json'))
    assert not os.path.exists(os.path.join(data_home, '3-maestro-v2.0.0.json'))

    if os.path.exists(data_home):
        shutil.rmtree(data_home)
    maestro.download(data_home=data_home, partial_download=['metadata', 'midi'])
    assert not os.path.exists(os.path.join(data_home, '1-maestro-v2.0.0.json'))
    assert os.path.exists(os.path.join(data_home, '2-maestro-v2.0.0.json'))
    assert not os.path.exists(os.path.join(data_home, '3-maestro-v2.0.0.json'))

    if os.path.exists(data_home):
        shutil.rmtree(data_home)
    maestro.download(data_home=data_home, partial_download=['metadata'])
    assert not os.path.exists(os.path.join(data_home, '1-maestro-v2.0.0.json'))
    assert not os.path.exists(os.path.join(data_home, '2-maestro-v2.0.0.json'))
    assert os.path.exists(os.path.join(data_home, '3-maestro-v2.0.0.json'))


def test_download(httpserver):
    data_home = 'tests/resources/mir_datasets/MAESTRO_download'
    if os.path.exists(data_home):
        shutil.rmtree(data_home)

    # download the full dataset
    httpserver.serve_content(
        open('tests/resources/download/maestro-v2.0.0.zip', 'rb').read()
    )

    maestro.REMOTES = {
        'all': download_utils.RemoteFileMetadata(
            filename='maestro-v2.0.0.zip',
            url=httpserver.url,
            checksum=('625180ffa41cd9f2ab7252dd954b9e8a'),
            destination_dir=None,
        )
    }
    maestro.download(data_home=data_home)

    assert os.path.exists(data_home)
    assert not os.path.exists(os.path.join(data_home, 'maestro-v2.0.0'))

    assert os.path.exists(os.path.join(data_home, "maestro-v2.0.0.json"))
    assert os.path.exists(
        os.path.join(
            data_home,
            "2004/MIDI-Unprocessed_XP_22_R2_2004_01_ORIG_MID--AUDIO_22_R2_2004_04_Track04_wav.wav",
        )
    )
    assert os.path.exists(
        os.path.join(
            data_home,
            "2004/MIDI-Unprocessed_XP_22_R2_2004_01_ORIG_MID--AUDIO_22_R2_2004_04_Track04_wav.midi",
        )
    )

    if os.path.exists(data_home):
        shutil.rmtree(data_home)

    # download the midi-only zip
    httpserver.serve_content(
        open('tests/resources/download/maestro-v2.0.0-midi.zip', 'rb').read()
    )

    maestro.REMOTES = {
        'midi': download_utils.RemoteFileMetadata(
            filename='maestro-v2.0.0-midi.zip',
            url=httpserver.url,
            checksum=('c82283fff347ed2bd833693c09a9f01d'),
            destination_dir=None,
        )
    }
    maestro.download(data_home=data_home, partial_download=['midi'])

    assert os.path.exists(data_home)
    assert not os.path.exists(os.path.join(data_home, 'maestro-v2.0.0'))

    assert os.path.exists(os.path.join(data_home, "maestro-v2.0.0.json"))
    assert not os.path.exists(
        os.path.join(
            data_home,
            "2004/MIDI-Unprocessed_XP_22_R2_2004_01_ORIG_MID--AUDIO_22_R2_2004_04_Track04_wav.wav",
        )
    )
    assert os.path.exists(
        os.path.join(
            data_home,
            "2004/MIDI-Unprocessed_XP_22_R2_2004_01_ORIG_MID--AUDIO_22_R2_2004_04_Track04_wav.midi",
        )
    )

    if os.path.exists(data_home):
        shutil.rmtree(data_home)

    # download only the metadata
    httpserver.serve_content(
        open('tests/resources/download/maestro-v2.0.0.json', 'rb').read()
    )

    maestro.REMOTES = {
        'metadata': download_utils.RemoteFileMetadata(
            filename='maestro-v2.0.0.json',
            url=httpserver.url,
            checksum=('d41d8cd98f00b204e9800998ecf8427e'),
            destination_dir=None,
        )
    }
    maestro.download(data_home=data_home, partial_download=['metadata'])

    assert os.path.exists(data_home)
    assert not os.path.exists(os.path.join(data_home, 'maestro-v2.0.0'))

    assert os.path.exists(os.path.join(data_home, "maestro-v2.0.0.json"))
    assert not os.path.exists(
        os.path.join(
            data_home,
            "2004/MIDI-Unprocessed_XP_22_R2_2004_01_ORIG_MID--AUDIO_22_R2_2004_04_Track04_wav.wav",
        )
    )
    assert not os.path.exists(
        os.path.join(
            data_home,
            "2004/MIDI-Unprocessed_XP_22_R2_2004_01_ORIG_MID--AUDIO_22_R2_2004_04_Track04_wav.midi",
        )
    )

    if os.path.exists(data_home):
        shutil.rmtree(data_home)



# -*- coding: utf-8 -*-


from .version import version as __version__

__all__ = [
    'beatles',
    'beatport_key',
    'dali',
    'giantsteps_tempo',
    'giantsteps_key',
    'groove_midi',
    'gtzan_genre',
    'guitarset',
    'ikala',
    'maestro',
    'medley_solos_db',
    'medleydb_melody',
    'medleydb_pitch',
    'mridangam_stroke',
    'orchset',
    'rwc_classical',
    'rwc_jazz',
    'rwc_popular',
    'salami',
    'tinysol',
]



# -*- coding: utf-8 -*-

from mirdata import rwc_jazz, utils
from tests.test_utils import run_track_tests


def test_track():

    default_trackid = 'RM-J004'
    data_home = 'tests/resources/mir_datasets/RWC-Jazz'
    track = rwc_jazz.Track(default_trackid, data_home=data_home)

    expected_attributes = {
        'track_id': 'RM-J004',
        'audio_path': 'tests/resources/mir_datasets/RWC-Jazz/'
        + 'audio/rwc-j-m01/4.wav',
        'sections_path': 'tests/resources/mir_datasets/RWC-Jazz/'
        + 'annotations/AIST.RWC-MDB-J-2001.CHORUS/RM-J004.CHORUS.TXT',
        'beats_path': 'tests/resources/mir_datasets/RWC-Jazz/'
        + 'annotations/AIST.RWC-MDB-J-2001.BEAT/RM-J004.BEAT.TXT',
        'piece_number': 'No. 4',
        'suffix': 'M01',
        'track_number': 'Tr. 04',
        'title': 'Crescent Serenade (Piano Solo)',
        'artist': 'Makoto Nakamura',
        'duration': 167,
        'variation': 'Instrumentation 1',
        'instruments': 'Pf',
    }

    expected_property_types = {'beats': utils.BeatData, 'sections': utils.SectionData}

    run_track_tests(track, expected_attributes, expected_property_types)

    # test audio loading functions
    y, sr = track.audio
    assert sr == 44100
    assert y.shape == (44100 * 2,)


def test_to_jams():

    data_home = 'tests/resources/mir_datasets/RWC-Jazz'
    track = rwc_jazz.Track('RM-J004', data_home=data_home)
    jam = track.to_jams()

    beats = jam.search(namespace='beat')[0]['data']
    assert [beat.time for beat in beats] == [
        0.05,
        0.86,
        1.67,
        2.48,
        3.29,
        4.1,
        4.91,
        5.72,
        6.53,
        7.34,
    ]
    assert [beat.duration for beat in beats] == [
        0.0,
        0.0,
        0.0,
        0.0,
        0.0,
        0.0,
        0.0,
        0.0,
        0.0,
        0.0,
    ]
    assert [beat.value for beat in beats] == [1, 2, 1, 2, 1, 2, 1, 2, 1, 2]
    assert [beat.confidence for beat in beats] == [
        None,
        None,
        None,
        None,
        None,
        None,
        None,
        None,
        None,
        None,
    ]

    segments = jam.search(namespace='segment')[0]['data']
    assert [segment.time for segment in segments] == [0.05, 6.53, 152.06]
    assert [segment.duration for segment in segments] == [
        6.48,
        13.099999999999998,
        13.319999999999993,
    ]
    assert [segment.value for segment in segments] == [
        'nothing',
        'chorus A',
        'chorus B',
    ]
    assert [segment.confidence for segment in segments] == [None, None, None]

    assert jam['file_metadata']['title'] == 'Crescent Serenade (Piano Solo)'
    assert jam['file_metadata']['artist'] == 'Makoto Nakamura'


def test_load_metadata():
    data_home = 'tests/resources/mir_datasets/RWC-Jazz'
    metadata = rwc_jazz._load_metadata(data_home)
    assert metadata['data_home'] == data_home
    assert metadata['RM-J004'] == {
        'piece_number': 'No. 4',
        'suffix': 'M01',
        'track_number': 'Tr. 04',
        'title': 'Crescent Serenade (Piano Solo)',
        'artist': 'Makoto Nakamura',
        'duration': 167,
        'variation': 'Instrumentation 1',
        'instruments': 'Pf',
    }
    assert metadata['RM-J044'] == {
        'piece_number': 'No. 44',
        'suffix': 'M04',
        'track_number': 'Tr. 09',
        'title': 'Joyful, Joyful, We Adore Thee',
        'artist': 'K’s Band',
        'duration': 270,
        'variation': 'Style (Free jazz)',
        'instruments': 'Pf & Bs & Dr & Gt & Ts & Fl & Bar',
    }

    metadata_none = rwc_jazz._load_metadata('asdf/asdf')
    assert metadata_none is None



# -*- coding: utf-8 -*-

import numpy as np

from mirdata import ikala, utils
from tests.test_utils import run_track_tests


def test_track():
    default_trackid = '10161_chorus'
    data_home = 'tests/resources/mir_datasets/iKala'
    track = ikala.Track(default_trackid, data_home=data_home)

    expected_attributes = {
        'track_id': '10161_chorus',
        'audio_path': 'tests/resources/mir_datasets/iKala/'
        + 'Wavfile/10161_chorus.wav',
        'song_id': '10161',
        'section': 'chorus',
        'singer_id': '1',
        'f0_path': 'tests/resources/mir_datasets/iKala/PitchLabel/10161_chorus.pv',
        'lyrics_path': 'tests/resources/mir_datasets/iKala/Lyrics/10161_chorus.lab',
    }

    expected_property_types = {'f0': utils.F0Data, 'lyrics': utils.LyricData}

    assert track._track_paths == {
        'audio': ['Wavfile/10161_chorus.wav', '278ae003cb0d323e99b9a643c0f2eeda'],
        'pitch': ['PitchLabel/10161_chorus.pv', '0d93a011a9e668fd80673049089bbb14'],
        'lyrics': ['Lyrics/10161_chorus.lab', '79bbeb72b422056fd43be4e8d63319ce'],
    }

    run_track_tests(track, expected_attributes, expected_property_types)

    # test audio loading functions
    vocal, sr_vocal = track.vocal_audio
    assert sr_vocal == 44100
    assert vocal.shape == (44100 * 2,)

    instrumental, sr_instrumental = track.instrumental_audio
    assert sr_instrumental == 44100
    assert instrumental.shape == (44100 * 2,)

    # make sure we loaded the correct channels to vocal/instrumental
    # (in this example, the first quarter second has only instrumentals)
    assert np.mean(np.abs(vocal[:8820])) < np.mean(np.abs(instrumental[:8820]))

    mix, sr_mix = track.mix_audio
    assert sr_mix == 44100
    assert mix.shape == (44100 * 2,)
    assert np.array_equal(mix, instrumental + vocal)


def test_to_jams():

    data_home = 'tests/resources/mir_datasets/iKala'
    track = ikala.Track('10161_chorus', data_home=data_home)
    jam = track.to_jams()

    lyrics = jam.search(namespace='lyric')[0]['data']
    assert [lyric.time for lyric in lyrics] == [0.027, 0.232]
    assert [lyric.duration for lyric in lyrics] == [0.20500000000000002, 0.736]
    assert [lyric.value for lyric in lyrics] == ['JUST', 'WANNA']
    assert [lyric.confidence for lyric in lyrics] == [None, None]

    f0s = jam.search(namespace='pitch_contour')[0]['data']
    assert [f0.time for f0 in f0s] == [0.016, 0.048]
    assert [f0.duration for f0 in f0s] == [0.0, 0.0]
    assert [f0.value for f0 in f0s] == [
        {'frequency': 0.0, 'index': 0, 'voiced': False},
        {'frequency': 260.946404518887, 'index': 0, 'voiced': True},
    ]
    assert [f0.confidence for f0 in f0s] == [0.0, 1.0]


def test_load_f0():
    # load a file which exists
    f0_path = 'tests/resources/mir_datasets/iKala/PitchLabel/10161_chorus.pv'
    f0_data = ikala.load_f0(f0_path)

    # check types
    assert type(f0_data) == utils.F0Data
    assert type(f0_data.times) is np.ndarray
    assert type(f0_data.frequencies) is np.ndarray
    assert type(f0_data.confidence) is np.ndarray

    # check values
    assert np.array_equal(f0_data.times, np.array([0.016, 0.048]))
    assert np.array_equal(f0_data.frequencies, np.array([0.0, 260.946404518887]))
    assert np.array_equal(f0_data.confidence, np.array([0.0, 1.0]))


def test_load_lyrics():
    # load a file without pronunciations
    lyrics_path_simple = 'tests/resources/mir_datasets/iKala/Lyrics/10161_chorus.lab'
    lyrics_data_simple = ikala.load_lyrics(lyrics_path_simple)

    # check types
    assert type(lyrics_data_simple) is utils.LyricData
    assert type(lyrics_data_simple.start_times) is np.ndarray
    assert type(lyrics_data_simple.end_times) is np.ndarray
    assert type(lyrics_data_simple.lyrics) is np.ndarray
    assert type(lyrics_data_simple.pronunciations) is np.ndarray

    # check values
    assert np.array_equal(lyrics_data_simple.start_times, np.array([0.027, 0.232]))
    assert np.array_equal(lyrics_data_simple.end_times, np.array([0.232, 0.968]))
    assert np.array_equal(lyrics_data_simple.lyrics, np.array(['JUST', 'WANNA']))
    assert np.array_equal(lyrics_data_simple.pronunciations, np.array([None, None]))

    # load a file with pronunciations
    lyrics_path_pronun = 'tests/resources/mir_datasets/iKala/Lyrics/10164_chorus.lab'
    lyrics_data_pronun = ikala.load_lyrics(lyrics_path_pronun)

    # check types
    assert type(lyrics_data_pronun) is utils.LyricData
    assert type(lyrics_data_pronun.start_times) is np.ndarray
    assert type(lyrics_data_pronun.end_times) is np.ndarray
    assert type(lyrics_data_pronun.lyrics) is np.ndarray
    assert type(lyrics_data_pronun.pronunciations) is np.ndarray

    # check values
    assert np.array_equal(lyrics_data_pronun.start_times, np.array([0.021, 0.571]))
    assert np.array_equal(lyrics_data_pronun.end_times, np.array([0.189, 1.415]))
    assert np.array_equal(lyrics_data_pronun.lyrics, np.array(['ASDF', 'EVERYBODY']))
    assert np.array_equal(lyrics_data_pronun.pronunciations, np.array(['t i au', None]))


def test_load_metadata():
    data_home = 'tests/resources/mir_datasets/iKala'
    metadata = ikala._load_metadata(data_home)
    assert metadata['data_home'] == data_home
    assert metadata['10161'] == '1'
    assert metadata['21025'] == '1'

    metadata_none = ikala._load_metadata('asdf/asdf')
    assert metadata_none is None



# -*- coding: utf-8 -*-
"""TinySOL Dataset Loader.

TinySOL is a dataset of 2913 samples, each containing a single musical note from one of 14
different instruments:

    Bass Tuba
    French Horn
    Trombone
    Trumpet in C
    Accordion
    Contrabass
    Violin
    Viola
    Violoncello
    Bassoon
    Clarinet in B-flat
    Flute
    Oboe
    Alto Saxophone


These sounds were originally recorded at Ircam in Paris (France) between 1996
and 1999, as part of a larger project named Studio On Line (SOL). Although SOL
contains many combinations of mutes and extended playing techniques, TinySOL
purely consists of sounds played in the so-called "ordinary" style, and in
absence of mute.

TinySOL can be used for education and research purposes. In particular, it can
be employed as a dataset for training and/or evaluating music information
retrieval (MIR) systems, for tasks such as instrument recognition or
fundamental frequency estimation. For this purpose, we provide an official 5-fold
split of TinySOL as a metadata attribute. This split has been carefully balanced
in terms of instrumentation, pitch range, and dynamics. For the sake of research
reproducibility, we encourage users of TinySOL to adopt this split and report
their results in terms of average performance across folds.

We encourage TinySOL users to subscribe to the Ircam Forum so that they can
have access to larger versions of SOL.

For more details, please visit: https://www.orch-idea.org/
"""

import csv
import librosa
import logging
import os

from mirdata import download_utils
from mirdata import jams_utils
from mirdata import track
from mirdata import utils

DATASET_DIR = "TinySOL"

REMOTES = {
    'audio': download_utils.RemoteFileMetadata(
        filename="TinySOL.tar.gz",
        url="https://zenodo.org/record/3685367/files/TinySOL.tar.gz?download=1",
        checksum="36030a7fe389da86c3419e5ee48e3b7f",
        destination_dir="audio",
    ),
    'annotations': download_utils.RemoteFileMetadata(
        filename="TinySOL_metadata.csv",
        url="https://zenodo.org/record/3685367/files/TinySOL_metadata.csv?download=1",
        checksum="a86c9bb115f69e61f2f25872e397fc4a",
        destination_dir="annotation",
    ),
}

STRING_ROMAN_NUMERALS = {1: "I", 2: "II", 3: "III", 4: "IV"}


def _load_metadata(data_home):
    metadata_path = os.path.join(data_home, "annotation", "TinySOL_metadata.csv")

    if not os.path.exists(metadata_path):
        logging.info("Metadata file {} not found.".format(metadata_path))
        return None

    metadata_index = {}
    with open(metadata_path, "r") as fhandle:
        csv_reader = csv.reader(fhandle, delimiter=",")
        next(csv_reader)
        for row in csv_reader:
            key = os.path.splitext(os.path.split(row[0])[1])[0]
            metadata_index[key] = {
                "Fold": int(row[1]),
                "Family": row[2],
                "Instrument (abbr.)": row[3],
                "Instrument (in full)": row[4],
                "Technique (abbr.)": row[5],
                "Technique (in full)": row[6],
                "Pitch": row[7],
                "Pitch ID": int(row[8]),
                "Dynamics": row[9],
                "Dynamics ID": int(row[10]),
                "Instance ID": int(row[11]),
                "Resampled": (row[13] == "TRUE"),
            }
            if len(row[12]) > 0:
                metadata_index[key]["String ID"] = int(float(row[12]))

    metadata_index["data_home"] = data_home

    return metadata_index


DATA = utils.LargeData("tinysol_index.json", _load_metadata)


class Track(track.Track):
    """tinysol Track class

    Args:
        track_id (str): track id of the track
        data_home (str): Local path where the dataset is stored. default=None
            If `None`, looks for the data in the default directory, `~/mir_datasets`

    Attributes:
        audio_path (str): path of the audio file
        dynamics (str): dynamics abbreviation. Ex: pp, mf, ff, etc.
        dynamics_id (int): pp=0, p=1, mf=2, f=3, ff=4
        family (str): instrument family encoded by its English name
        instance_id (int): instance ID. Either equal to 0, 1, 2, or 3.
        instrument_abbr (str): instrument abbreviation
        instrument_full (str): instrument encoded by its English name
        is_resampled (bool): True if this sample was pitch-shifted from a neighbor; False if it was genuinely recorded.
        pitch (str): string containing English pitch class and octave number
        pitch_id (int): MIDI note index, where middle C ("C4") corresponds to 60
        string_id (NoneType): string ID. By musical convention, the first
            string is the highest. On wind instruments, this is replaced by `None`.
        technique_abbr (str): playing technique abbreviation
        technique_full (str): playing technique encoded by its English name
        track_id (str): track id

    """

    def __init__(self, track_id, data_home=None):
        if track_id not in DATA.index:
            raise ValueError("{} is not a valid track ID in TinySOL".format(track_id))

        self.track_id = track_id

        if data_home is None:
            data_home = utils.get_default_dataset_path(DATASET_DIR)

        self._data_home = data_home
        self._track_paths = DATA.index[track_id]

        metadata = DATA.metadata(data_home)
        if metadata is not None and track_id in metadata:
            self._track_metadata = metadata[track_id]
        else:
            self._track_metadata = {
                "Family": None,
                "Instrument (abbr.)": None,
                "Instrument (in full)": None,
                "Technique (abbr.)": None,
                "Technique (in full)": None,
                "Pitch": None,
                "Pitch ID": None,
                "Dynamics": None,
                "Dynamics ID": None,
                "Instance ID": None,
                "String ID": None,
                "Resampled": None,
            }

        self.audio_path = os.path.join(
            self._data_home, "audio", self._track_paths["audio"][0]
        )

        self.family = self._track_metadata["Family"]
        self.instrument_abbr = self._track_metadata["Instrument (abbr.)"]
        self.instrument_full = self._track_metadata["Instrument (in full)"]
        self.technique_abbr = self._track_metadata["Technique (abbr.)"]
        self.technique_full = self._track_metadata["Technique (in full)"]
        self.pitch = self._track_metadata["Pitch"]
        self.pitch_id = self._track_metadata["Pitch ID"]
        self.dynamics = self._track_metadata["Dynamics"]
        self.dynamics_id = self._track_metadata["Dynamics ID"]
        self.instance_id = self._track_metadata["Instance ID"]
        if "String ID" in self._track_metadata:
            self.string_id = self._track_metadata["String ID"]
        else:
            self.string_id = None
        self.is_resampled = self._track_metadata["Resampled"]

    @property
    def audio(self):
        """(np.ndarray, float): audio signal, sample rate"""
        return load_audio(self.audio_path)

    def to_jams(self):
        """Jams: the track's data in jams format"""
        return jams_utils.jams_converter(
            audio_path=self.audio_path, metadata=self._track_metadata
        )


def load_audio(audio_path):
    """Load a TinySOL audio file.

    Args:
        audio_path (str): path to audio file

    Returns:
        y (np.ndarray): the mono audio signal
        sr (float): The sample rate of the audio file

    """
    if not os.path.exists(audio_path):
        raise IOError("audio_path {} does not exist".format(audio_path))

    return librosa.load(audio_path, sr=None, mono=True)


def download(
    data_home=None, partial_download=None, force_overwrite=False, cleanup=True
):
    """Download TinySOL.

    Args:
        data_home (str):
            Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`
        force_overwrite (bool):
            Whether to overwrite the existing downloaded data
        partial_download (list):
            List indicating what to partially download. The list can include any of:
            * `'annotations'` the annotation files
            * `'audio'` the audio files
            If `None`, all data is downloaded.

        cleanup (bool):
            Whether to delete the zip/tar file after extracting.

    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    download_utils.downloader(
        data_home,
        remotes=REMOTES,
        partial_download=partial_download,
        info_message=None,
        force_overwrite=force_overwrite,
        cleanup=cleanup,
    )


def track_ids():
    """Return track ids

    Returns:
        (list): A list of track ids
    """
    return list(DATA.index.keys())


def validate(data_home=None, silence=False):
    """Validate if the stored dataset is a valid version

    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`

    Returns:
        missing_files (list): List of file paths that are in the dataset index
            but missing locally
        invalid_checksums (list): List of file paths that file exists in the dataset
            index but has a different checksum compare to the reference checksum
    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    missing_files, invalid_checksums = utils.validator(
        DATA.index, os.path.join(data_home, "audio"), silence=silence
    )
    return missing_files, invalid_checksums


def load(data_home=None):
    """Load TinySOL
    Args:
        data_home (str): Local path where TinySOL is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`
    Returns:
        (dict): {`track_id`: track data}
    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    tinysol_data = {}
    for key in DATA.index.keys():
        tinysol_data[key] = Track(key, data_home=data_home)
    return tinysol_data


def cite():
    """Print the reference"""

    cite_data = """
=========== MLA ===========
Cella, Carmine Emanuele, et al., "OrchideaSOL: A dataset of extended
instrumental techniques for computer-aided orchestration". Under review, 2020.

========== Bibtex ==========
@inproceedings{cella2020preprint,
author={Cella, Carmine Emanuele and Ghisi, Daniele and Lostanlen, Vincent and
Lévy, Fabien and Fineberg, Joshua and Maresz, Yan},
title={{OrchideaSOL}: {A} dataset of extended
instrumental techniques for computer-aided orchestration},
bootktitle={Under review},
year={2020}}
"""
    print(cite_data)



# -*- coding: utf-8 -*-

import numpy as np
from mirdata import salami, utils
from tests.test_utils import run_track_tests


def test_track():

    default_trackid = '2'
    data_home = 'tests/resources/mir_datasets/Salami'
    track = salami.Track(default_trackid, data_home=data_home)

    expected_attributes = {
        'track_id': '2',
        'audio_path': 'tests/resources/mir_datasets/Salami/' + 'audio/2.mp3',
        'sections_annotator1_uppercase_path': 'tests/resources/mir_datasets/Salami/'
        + 'salami-data-public-hierarchy-corrections/annotations/2/parsed/textfile1_uppercase.txt',
        'sections_annotator1_lowercase_path': 'tests/resources/mir_datasets/Salami/'
        + 'salami-data-public-hierarchy-corrections/annotations/2/parsed/textfile1_lowercase.txt',
        'sections_annotator2_uppercase_path': 'tests/resources/mir_datasets/Salami/'
        + 'salami-data-public-hierarchy-corrections/annotations/2/parsed/textfile2_uppercase.txt',
        'sections_annotator2_lowercase_path': 'tests/resources/mir_datasets/Salami/'
        + 'salami-data-public-hierarchy-corrections/annotations/2/parsed/textfile2_lowercase.txt',
        'source': 'Codaich',
        'annotator_1_id': '5',
        'annotator_2_id': '8',
        'duration': 264,
        'title': 'For_God_And_Country',
        'artist': 'The_Smashing_Pumpkins',
        'annotator_1_time': '37',
        'annotator_2_time': '45',
        'broad_genre': 'popular',
        'genre': 'Alternative_Pop___Rock',
    }

    expected_property_types = {
        'sections_annotator_1_uppercase': utils.SectionData,
        'sections_annotator_1_lowercase': utils.SectionData,
        'sections_annotator_2_uppercase': utils.SectionData,
        'sections_annotator_2_lowercase': utils.SectionData,
    }

    run_track_tests(track, expected_attributes, expected_property_types)

    # test audio loading functions
    y, sr = track.audio
    assert sr == 44100
    assert y.shape == (89856,)

    # Test file with missing annotations
    track = salami.Track('192', data_home=data_home)

    # test attributes
    assert track.source == 'Codaich'
    assert track.annotator_1_id == '16'
    assert track.annotator_2_id == '14'
    assert track.duration == 209
    assert track.title == 'Sull__aria'
    assert track.artist == 'Compilations'
    assert track.annotator_1_time == '20'
    assert track.annotator_2_time == ''
    assert track.broad_genre == 'classical'
    assert track.genre == 'Classical_-_Classical'
    assert track.track_id == '192'
    assert track._data_home == data_home

    assert track._track_paths == {
        'audio': ['audio/192.mp3', 'd954d5dc9f17d66155d3310d838756b8'],
        'annotator_1_uppercase': [
            'salami-data-public-hierarchy-corrections/annotations/192/parsed/textfile1_uppercase.txt',
            '4d268cfd27fe011dbe579f25f8d125ce',
        ],
        'annotator_1_lowercase': [
            'salami-data-public-hierarchy-corrections/annotations/192/parsed/textfile1_lowercase.txt',
            '6640237e7844d0d9d37bf21cf96a2690',
        ],
        'annotator_2_uppercase': [None, None],
        'annotator_2_lowercase': [None, None],
    }

    # test that cached properties don't fail and have the expected type
    assert type(track.sections_annotator_1_uppercase) is utils.SectionData
    assert type(track.sections_annotator_1_lowercase) is utils.SectionData
    assert track.sections_annotator_2_uppercase is None
    assert track.sections_annotator_2_lowercase is None

    # Test file with missing annotations
    track = salami.Track('1015', data_home=data_home)

    assert track._track_paths == {
        'audio': ['audio/1015.mp3', '811a4a6b46f0c15a61bfb299b21ebdc4'],
        'annotator_1_uppercase': [None, None],
        'annotator_1_lowercase': [None, None],
        'annotator_2_uppercase': [
            'salami-data-public-hierarchy-corrections/annotations/1015/parsed/textfile2_uppercase.txt',
            'e4a268342a45fdffd8ec9c3b8287ad8b',
        ],
        'annotator_2_lowercase': [
            'salami-data-public-hierarchy-corrections/annotations/1015/parsed/textfile2_lowercase.txt',
            '201642fcea4a27c60f7b48de46a82234',
        ],
    }

    # test that cached properties don't fail and have the expected type
    assert track.sections_annotator_1_uppercase is None
    assert track.sections_annotator_1_lowercase is None
    assert type(track.sections_annotator_2_uppercase) is utils.SectionData
    assert type(track.sections_annotator_2_lowercase) is utils.SectionData


def test_to_jams():

    data_home = 'tests/resources/mir_datasets/Salami'
    track = salami.Track('2', data_home=data_home)
    jam = track.to_jams()

    segments = jam.search(namespace='multi_segment')[0]['data']
    assert [segment.time for segment in segments] == [
        0.0,
        0.0,
        0.464399092,
        0.464399092,
        5.191269841,
        14.379863945,
        254.821632653,
        258.900453514,
        263.205419501,
        263.205419501,
    ]
    assert [segment.duration for segment in segments] == [
        0.464399092,
        0.464399092,
        13.915464853,
        4.726870749000001,
        249.630362812,
        248.82555555599998,
        4.078820860999997,
        4.304965987000003,
        1.6797959180000248,
        1.6797959180000248,
    ]
    assert [segment.value for segment in segments] == [
        {'label': 'Silence', 'level': 0},
        {'label': 'Silence', 'level': 1},
        {'label': 'A', 'level': 0},
        {'label': 'b', 'level': 1},
        {'label': 'b', 'level': 1},
        {'label': 'B', 'level': 0},
        {'label': 'ab', 'level': 1},
        {'label': 'ab', 'level': 1},
        {'label': 'Silence', 'level': 0},
        {'label': 'Silence', 'level': 1},
    ]
    assert [segment.confidence for segment in segments] == [
        None,
        None,
        None,
        None,
        None,
        None,
        None,
        None,
        None,
        None,
    ]

    assert jam['file_metadata']['title'] == 'For_God_And_Country'
    assert jam['file_metadata']['artist'] == 'The_Smashing_Pumpkins'


def test_load_sections():
    # load a file which exists
    sections_path = (
        'tests/resources/mir_datasets/Salami/'
        + 'salami-data-public-hierarchy-corrections/annotations/2/parsed/textfile1_uppercase.txt'
    )
    section_data = salami.load_sections(sections_path)

    # check types
    assert type(section_data) == utils.SectionData
    assert type(section_data.intervals) is np.ndarray
    assert type(section_data.labels) is list

    # check valuess
    assert np.array_equal(
        section_data.intervals[:, 0],
        np.array([0.0, 0.464399092, 14.379863945, 263.205419501]),
    )
    assert np.array_equal(
        section_data.intervals[:, 1],
        np.array([0.464399092, 14.379863945, 263.205419501, 264.885215419]),
    )
    assert np.array_equal(
        section_data.labels, np.array(['Silence', 'A', 'B', 'Silence'])
    )

    # load none
    section_data_none = salami.load_sections(None)
    assert section_data_none is None


def test_load_metadata():
    data_home = 'tests/resources/mir_datasets/Salami'
    metadata = salami._load_metadata(data_home)
    assert metadata['data_home'] == data_home
    assert metadata['2'] == {
        'source': 'Codaich',
        'annotator_1_id': '5',
        'annotator_2_id': '8',
        'duration': 264,
        'title': 'For_God_And_Country',
        'artist': 'The_Smashing_Pumpkins',
        'annotator_1_time': '37',
        'annotator_2_time': '45',
        'class': 'popular',
        'genre': 'Alternative_Pop___Rock',
    }

    none_metadata = salami._load_metadata('asdf/asdf')
    assert none_metadata is None



# -*- coding: utf-8 -*-
"""
This test takes a long time, but it makes sure that the datset can be locally downloaded,
validated successfully, and loaded.
"""
import importlib
from tests.test_utils import get_attributes_and_properties
import os
import pytest

import mirdata


@pytest.fixture()
def dataset(test_dataset):
    if test_dataset == '':
        return None
    elif test_dataset not in mirdata.__all__:
        raise ValueError("{} is not a dataset in mirdata".format(test_dataset))

    return importlib.import_module("mirdata.{}".format(test_dataset))


@pytest.fixture()
def data_home_dir(dataset):
    if dataset is None:
        return None
    return os.path.join('tests/resources/mir_datasets_full', dataset.DATASET_DIR)


# This is magically skipped by the the remote fixture `skip_remote` in conftest.py
# when tests are run without the --local flag
def test_download(skip_remote, dataset, data_home_dir, skip_download):
    if dataset is None:
        pytest.skip()

    # download the dataset
    if not skip_download:
        dataset.download(data_home=data_home_dir)

        print(
            "If this dataset does not have openly downloadable data, "
            + "follow the instructions printed by the download message and "
            + "rerun this test."
        )


def test_validation(skip_remote, dataset, data_home_dir):
    if dataset is None:
        pytest.skip()

    # run validation
    missing_files, invalid_checksums = dataset.validate(
        data_home=data_home_dir, silence=True
    )

    assert missing_files == {}
    assert invalid_checksums == {}


def test_load(skip_remote, dataset, data_home_dir):
    if dataset is None:
        pytest.skip()

    # run load
    all_data = dataset.load(data_home=data_home_dir)

    assert isinstance(all_data, dict)

    track_ids = dataset.track_ids()
    assert set(track_ids) == set(all_data.keys())

    # test that all attributes and properties can be called
    for track_id in track_ids:
        track = all_data[track_id]
        track_data = get_attributes_and_properties(track)

        for attr in track_data['attributes']:
            ret = getattr(track, attr)

        for prop in track_data['properties']:
            ret = getattr(track, prop)

        for cprop in track_data['cached_properties']:
            ret = getattr(track, cprop)

        jam = track.to_jams()
        assert jam.validate()



# -*- coding: utf-8 -*-

from mirdata import medley_solos_db
from tests.test_utils import run_track_tests


def test_track():
    default_trackid = 'd07b1fc0-567d-52c2-fef4-239f31c9d40e'
    data_home = 'tests/resources/mir_datasets/Medley-solos-DB'
    track = medley_solos_db.Track(default_trackid, data_home=data_home)

    expected_attributes = {
        'track_id': 'd07b1fc0-567d-52c2-fef4-239f31c9d40e',
        'audio_path': 'tests/resources/mir_datasets/Medley-solos-DB/'
        + 'audio/Medley-solos-DB_validation-3_d07b1fc0-567d-52c2-fef4-239f31c9d40e.wav',
        'instrument': 'flute',
        'instrument_id': 3,
        'song_id': 210,
        'subset': 'validation',
    }

    expected_property_types = {}

    run_track_tests(track, expected_attributes, expected_property_types)

    y, sr = track.audio
    assert y.shape == (65536,)
    assert sr == 22050


def test_to_jams():

    data_home = 'tests/resources/mir_datasets/Medley-solos-DB'
    track = medley_solos_db.Track(
        'd07b1fc0-567d-52c2-fef4-239f31c9d40e', data_home=data_home
    )
    jam = track.to_jams()

    assert jam['sandbox']['instrument'] == 'flute'
    assert jam['sandbox']['instrument_id'] == 3
    assert jam['sandbox']['song_id'] == 210
    assert jam['sandbox']['subset'] == 'validation'



# -*- coding: utf-8 -*-

import numpy as np

from mirdata import medleydb_melody, utils
from tests.test_utils import run_track_tests


def test_track():
    default_trackid = 'MusicDelta_Beethoven'
    data_home = 'tests/resources/mir_datasets/MedleyDB-Melody'
    track = medleydb_melody.Track(default_trackid, data_home=data_home)

    expected_attributes = {
        'track_id': 'MusicDelta_Beethoven',
        'audio_path': 'tests/resources/mir_datasets/'
        + 'MedleyDB-Melody/audio/MusicDelta_Beethoven_MIX.wav',
        'melody1_path': 'tests/resources/mir_datasets/'
        + 'MedleyDB-Melody/melody1/MusicDelta_Beethoven_MELODY1.csv',
        'melody2_path': 'tests/resources/mir_datasets/'
        + 'MedleyDB-Melody/melody2/MusicDelta_Beethoven_MELODY2.csv',
        'melody3_path': 'tests/resources/mir_datasets/'
        + 'MedleyDB-Melody/melody3/MusicDelta_Beethoven_MELODY3.csv',
        'artist': 'MusicDelta',
        'title': 'Beethoven',
        'genre': 'Classical',
        'is_excerpt': True,
        'is_instrumental': True,
        'n_sources': 18,
    }

    expected_property_types = {
        'melody1': utils.F0Data,
        'melody2': utils.F0Data,
        'melody3': utils.MultipitchData,
    }

    run_track_tests(track, expected_attributes, expected_property_types)

    y, sr = track.audio
    assert sr == 44100
    assert y.shape == (44100 * 2,)


def test_to_jams():

    data_home = 'tests/resources/mir_datasets/MedleyDB-Melody'
    track = medleydb_melody.Track('MusicDelta_Beethoven', data_home=data_home)
    jam = track.to_jams()

    f0s = jam.search(namespace='pitch_contour')[1]['data']
    assert [f0.time for f0 in f0s] == [0.046439909297052155, 0.052244897959183675]
    assert [f0.duration for f0 in f0s] == [0.0, 0.0]
    assert [f0.value for f0 in f0s] == [
        {'frequency': 0.0, 'index': 0, 'voiced': False},
        {'frequency': 965.992, 'index': 0, 'voiced': True},
    ]
    assert [f0.confidence for f0 in f0s] == [0.0, 1.0]

    assert jam['file_metadata']['title'] == 'Beethoven'
    assert jam['file_metadata']['artist'] == 'MusicDelta'


def test_load_melody():
    # load a file which exists
    melody_path = (
        'tests/resources/mir_datasets/MedleyDB-Melody/'
        + 'melody1/MusicDelta_Beethoven_MELODY1.csv'
    )
    melody_data = medleydb_melody.load_melody(melody_path)

    # check types
    assert type(melody_data) == utils.F0Data
    assert type(melody_data.times) is np.ndarray
    assert type(melody_data.frequencies) is np.ndarray
    assert type(melody_data.confidence) is np.ndarray

    # check values
    assert np.array_equal(
        melody_data.times, np.array([0.0058049886621315194, 0.052244897959183675])
    )
    assert np.array_equal(melody_data.frequencies, np.array([0.0, 965.99199999999996]))
    assert np.array_equal(melody_data.confidence, np.array([0.0, 1.0]))


def test_load_melody3():
    # load a file which exists
    melody_path = (
        'tests/resources/mir_datasets/MedleyDB-Melody/'
        + 'melody3/MusicDelta_Beethoven_MELODY3.csv'
    )
    melody_data = medleydb_melody.load_melody3(melody_path)

    # check types
    assert type(melody_data) == utils.MultipitchData
    assert type(melody_data.times) is np.ndarray
    assert type(melody_data.frequency_list) is list
    assert type(melody_data.confidence_list) is list

    # check values
    assert np.array_equal(
        melody_data.times,
        np.array([0.046439909297052155, 0.052244897959183675, 0.1219047619047619]),
    )
    assert melody_data.frequency_list == [
        [0.0, 0.0, 497.01600000000002, 0.0, 0.0],
        [965.99199999999996, 996.46799999999996, 497.10599999999999, 0.0, 0.0],
        [
            987.32000000000005,
            987.93200000000002,
            495.46800000000002,
            495.29899999999998,
            242.98699999999999,
        ],
    ]

    assert melody_data.confidence_list == [
        [0.0, 0.0, 1.0, 0.0, 0.0],
        [1.0, 1.0, 1.0, 0.0, 0.0],
        [1.0, 1.0, 1.0, 1.0, 1.0],
    ]


def test_load_metadata():
    data_home = 'tests/resources/mir_datasets/MedleyDB-Melody'
    metadata = medleydb_melody._load_metadata(data_home)
    assert metadata['data_home'] == data_home
    assert metadata['MusicDelta_Beethoven'] == {
        'audio_path': 'MedleyDB-Melody/audio/MusicDelta_Beethoven_MIX.wav',
        'melody1_path': 'MedleyDB-Melody/melody1/MusicDelta_Beethoven_MELODY1.csv',
        'melody2_path': 'MedleyDB-Melody/melody2/MusicDelta_Beethoven_MELODY2.csv',
        'melody3_path': 'MedleyDB-Melody/melody3/MusicDelta_Beethoven_MELODY3.csv',
        'artist': 'MusicDelta',
        'title': 'Beethoven',
        'genre': 'Classical',
        'is_excerpt': True,
        'is_instrumental': True,
        'n_sources': 18,
    }

    metadata_none = medleydb_melody._load_metadata('asdf/asdf')
    assert metadata_none is None



import numpy as np
import DALI

from mirdata import dali, utils
from tests.test_utils import run_track_tests


def test_track():

    default_trackid = '4b196e6c99574dd49ad00d56e132712b'
    data_home = 'tests/resources/mir_datasets/DALI'
    track = dali.Track(default_trackid, data_home=data_home)

    expected_attributes = {
        'album': 'Mezmerize',
        'annotation_path': 'tests/resources/mir_datasets/DALI/'
        + 'annotations/4b196e6c99574dd49ad00d56e132712b.gz',
        'artist': 'System Of A Down',
        'audio_path': 'tests/resources/mir_datasets/DALI/'
        + 'audio/4b196e6c99574dd49ad00d56e132712b.mp3',
        'audio_url': 'zUzd9KyIDrM',
        'dataset_version': 1,
        'ground_truth': False,
        'language': 'english',
        'release_date': '2005',
        'scores_manual': 0,
        'scores_ncc': 0.9644769596900552,
        'title': 'B.Y.O.B.',
        'track_id': '4b196e6c99574dd49ad00d56e132712b',
        'url_working': True,
    }

    expected_property_types = {
        'notes': utils.NoteData,
        'words': utils.LyricData,
        'lines': utils.LyricData,
        'paragraphs': utils.LyricData,
        'annotation_object': DALI.Annotations,
    }

    run_track_tests(track, expected_attributes, expected_property_types)

    path_save = '/home/mfuentes/astre/code/repositories/mirdata/tests/resources/mir_datasets/DALI/annotations'
    name = 'test'
    track.annotation_object.write_json(path_save, name)

    audio, sr = track.audio
    assert sr == 48000
    assert audio.shape == (94208,)


def test_load_notes():
    notes_path = (
        'tests/resources/mir_datasets/DALI/annotations/'
        + '4b196e6c99574dd49ad00d56e132712b.gz'
    )
    note_data = dali.load_annotations_granularity(notes_path, 'notes')

    assert type(note_data) == utils.NoteData
    assert type(note_data.intervals) == np.ndarray
    assert type(note_data.notes) == np.ndarray

    assert np.array_equal(note_data.intervals[:, 0], np.array([24.125, 24.273, 24.420]))
    assert np.array_equal(note_data.intervals[:, 1], np.array([24.273, 24.420, 24.568]))
    assert np.array_equal(note_data.notes, np.array([1108.731, 1108.731, 1108.731]))


def test_load_words():
    data_path = (
        'tests/resources/mir_datasets/DALI/annotations/'
        + '4b196e6c99574dd49ad00d56e132712b.gz'
    )
    word_data = dali.load_annotations_granularity(data_path, 'words')

    assert type(word_data) == utils.LyricData
    assert type(word_data.start_times) == np.ndarray
    assert type(word_data.end_times) == np.ndarray
    assert type(word_data.lyrics) == np.ndarray

    assert np.array_equal(word_data.start_times, np.array([24.125, 24.273, 24.42]))
    assert np.array_equal(word_data.end_times, np.array([24.273, 24.42, 24.568]))
    assert np.array_equal(word_data.lyrics, np.array(['why', 'do', 'they']))


def test_load_lines():
    data_path = (
        'tests/resources/mir_datasets/DALI/annotations/'
        + '4b196e6c99574dd49ad00d56e132712b.gz'
    )
    line_data = dali.load_annotations_granularity(data_path, 'lines')

    assert type(line_data) == utils.LyricData
    assert type(line_data.start_times) == np.ndarray
    assert type(line_data.end_times) == np.ndarray
    assert type(line_data.lyrics) == np.ndarray

    print(line_data.start_times)
    print(line_data.end_times)
    print(line_data.lyrics)

    assert np.array_equal(line_data.start_times, np.array([24.125, 24.42]))
    assert np.array_equal(line_data.end_times, np.array([24.42, 24.568]))
    assert np.array_equal(line_data.lyrics, np.array(['why do', 'they']))


def test_load_paragraphs():
    data_path = (
        'tests/resources/mir_datasets/DALI/annotations/'
        + '4b196e6c99574dd49ad00d56e132712b.gz'
    )
    par_data = dali.load_annotations_granularity(data_path, 'paragraphs')

    assert type(par_data) == utils.LyricData
    assert type(par_data.start_times) == np.ndarray
    assert type(par_data.end_times) == np.ndarray
    assert type(par_data.lyrics) == np.ndarray

    assert np.array_equal(par_data.start_times, np.array([24.125, 24.420]))
    assert np.array_equal(par_data.end_times, np.array([24.420, 24.568]))
    assert np.array_equal(par_data.lyrics, np.array(['why do', 'they']))


def test_load_dali_object():
    data_path = (
        'tests/resources/mir_datasets/DALI/annotations/'
        + '4b196e6c99574dd49ad00d56e132712b.gz'
    )
    dali_data = dali.load_annotations_class(data_path)

    assert type(dali_data) == DALI.Annotations
    assert dali_data.annotations['annot']['notes'] == [
        {
            'text': 'why',
            'freq': [1108.7305239074883, 1108.7305239074883],
            'time': [24.12471002069169, 24.272507833284063],
            'index': 0,
        },
        {
            'text': 'do',
            'freq': [1108.7305239074883, 1108.7305239074883],
            'time': [24.272507833284063, 24.42030564587644],
            'index': 1,
        },
        {
            'text': 'they',
            'freq': [1108.7305239074883, 1108.7305239074883],
            'time': [24.42030564587644, 24.568103458468812],
            'index': 2,
        },
    ]
    assert dali_data.annotations['annot']['words'] == [
        {
            'text': 'why',
            'freq': [1108.7305239074883, 1108.7305239074883],
            'time': [24.12471002069169, 24.272507833284063],
            'index': 0,
        },
        {
            'text': 'do',
            'freq': [1108.7305239074883, 1108.7305239074883],
            'time': [24.272507833284063, 24.42030564587644],
            'index': 0,
        },
        {
            'text': 'they',
            'freq': [1108.7305239074883, 1108.7305239074883],
            'time': [24.42030564587644, 24.568103458468812],
            'index': 1,
        },
    ]
    assert dali_data.annotations['annot']['lines'] == [
        {
            'text': 'why do',
            'freq': [1108.7305239074883, 1108.7305239074883],
            'time': [24.12471002069169, 24.42030564587644],
            'index': 0,
        },
        {
            'text': 'they',
            'freq': [1108.7305239074883, 1108.7305239074883],
            'time': [24.42030564587644, 24.568103458468812],
            'index': 1,
        },
    ]
    assert dali_data.annotations['annot']['paragraphs'] == [
        {
            'text': 'why do',
            'freq': [1108.7305239074883, 1108.7305239074883],
            'time': [24.12471002069169, 24.42030564587644],
        },
        {
            'text': 'they',
            'freq': [1108.7305239074883, 1108.7305239074883],
            'time': [24.42030564587644, 24.568103458468812],
        },
    ]
